@@ -18,7 +18,8 @@ link_targets[2] = [
  ["(mod-path._.'ffi/objc.')", "../foreign/Objective-C_FFI.html#%28mod-path._ffi%2Fobjc%29"],
  ["(mod-path._.'ffi/unsafe.')", "../foreign/index.html"],
  ["(mod-path._.'ffi/unsafe/alloc.')", "../foreign/Allocation_and_Finalization.html"],
- ["(mod-path._.'ffi/unsafe/global.')", "../foreign/Process-Wide_Registration.html"],
+ ["(mod-path._.'ffi/unsafe/global.')", "../foreign/unsafe-global.html"],
+ ["(mod-path._.'ffi/unsafe/schedule.')", "../foreign/Thread_Scheduling.html"],
  ["(mod-path._.'ffi/unsafe/try-atomic.')", "../foreign/Speculatively_Atomic_Execution.html"],
  ["(mod-path._.'file/gunzip.')", "../file/gunzip.html"],
  ["(mod-path._.'file/ico.')", "../file/ico.html"],
@@ -105,6 +106,8 @@ link_targets[2] = [
  ["(mod-path._.'rnrs/lists-6.')", "../r6rs/R6RS_Libraries.html#%28mod-path._rnrs%2Flists-6%29"],
  ["(mod-path._.'rnrs/programs-6.')", "../r6rs/R6RS_Libraries.html#%28mod-path._rnrs%2Fprograms-6%29"],
  ["(mod-path._.'rnrs/records/inspection-6.')", "../r6rs/R6RS_Libraries.html#%28mod-path._rnrs%2Frecords%2Finspection-6%29"],
+ ["(mod-path._.'sasl/plain.')", "../sasl/index.html#%28mod-path._sasl%2Fplain%29"],
+ ["(mod-path._.'sasl/scram.')", "../sasl/index.html#%28mod-path._sasl%2Fscram%29"],
  ["(mod-path._.'scheme/cmdline.')", "../scheme/scheme_cmdline.html"],
  ["(mod-path._.'scheme/contract.')", "../scheme/scheme_contract.html"],
  ["(mod-path._.'scheme/dict.')", "../scheme/scheme_dict.html"],
@@ -127,7 +130,7 @@ link_targets[2] = [
  ["(mod-path._.'slideshow/text.')", "../slideshow/Text_Formatting_Helpers.html"],
  ["(mod-path._.'srfi/14.')", "../srfi/srfi-14.html"],
  ["(mod-path._.'srfi/23.')", "../srfi/srfi-23.html"],
- ["(mod-path._.'srfi/29.')", "../srfi/srfi-29.html"],
+ ["(mod-path._.'srfi/29.')", "../srfi-nf/srfi-29.html"],
  ["(mod-path._.'srfi/38.')", "../srfi/srfi-38.html"],
  ["(mod-path._.'srfi/41.')", "../srfi/srfi-41.html"],
  ["(mod-path._.'srfi/74.')", "../srfi/srfi-74.html"],
@@ -135,6 +138,7 @@ link_targets[2] = [
  ["(mod-path._.'srfi/98.')", "../srfi/srfi-98.html"],
  ["(mod-path._.'syntax-color/default-lexer.')", "../syntax-color/index.html#%28mod-path._syntax-color%2Fdefault-lexer%29"],
  ["(mod-path._.'syntax-color/token-tree.')", "../syntax-color/index.html#%28mod-path._syntax-color%2Ftoken-tree%29"],
+ ["(mod-path._.'syntax/apply-transformer.')", "../syntax/transformer-helpers.html#%28mod-path._syntax%2Fapply-transformer%29"],
  ["(mod-path._.'syntax/flatten-begin.')", "../syntax/transformer-helpers.html#%28mod-path._syntax%2Fflatten-begin%29"],
  ["(mod-path._.'syntax/id-set.')", "../syntax/syntax-helpers.html#%28mod-path._syntax%2Fid-set%29"],
  ["(mod-path._.'syntax/modresolve.')", "../syntax/module-helpers.html#%28mod-path._syntax%2Fmodresolve%29"],
@@ -398,6 +402,7 @@ link_targets[2] = [
  ["(part._(.'(lib._redex/redex..scrbl).'._.'sec~3ab~3alet-poly.'))", "../redex/benchmark.html#%28part._sec~3ab~3alet-poly%29"],
  ["(part._(.'(lib._redex/redex..scrbl).'._.'top.'))", "../redex/index.html"],
  ["(part._(.'(lib._redex/redex..scrbl).'._.'tue-mor.'))", "../redex/redex2015.html#%28part._tue-mor%29"],
+ ["(part._(.'(lib._sasl/sasl..scrbl).'._.'.S.A.S.L__.Simple_.Authentication_and_.Security_.Layer.'))", "../sasl/index.html"],
  ["(part._(.'(lib._scribblings/distributed-places/distributed-places..scrbl).'._.'distributed-places.'))", "../distributed-places/index.html"],
  ["(part._(.'(lib._scribblings/draw/draw..scrbl).'._.'.Alpha_.Channels_and_.Alpha_.Blending.'))", "../draw/overview.html#%28part._.Alpha_.Channels_and_.Alpha_.Blending%29"],
  ["(part._(.'(lib._scribblings/draw/draw..scrbl).'._.'.Drawing_.Contracts.'))", "../draw/Drawing_Contracts.html"],
@@ -452,6 +457,7 @@ link_targets[2] = [
  ["(part._(.'(lib._scribblings/gui/gui..scrbl).'._.'tb~3amiaoverview.'))", "../gui/editor-overview.html#%28part._tb~3amiaoverview%29"],
  ["(part._(.'(lib._scribblings/gui/gui..scrbl).'._.'text_.'))", "../gui/text_.html"],
  ["(part._(.'(lib._scribblings/guide/guide..scrbl).'._.'.A_.Dictionary.'))", "../guide/contracts-examples.html#%28part._.A_.Dictionary%29"],
+ ["(part._(.'(lib._scribblings/guide/guide..scrbl).'._.'.Building_.Your_.Own_.Synchronization_.Patterns.'))", "../guide/concurrency.html#%28part._.Building_.Your_.Own_.Synchronization_.Patterns%29"],
  ["(part._(.'(lib._scribblings/guide/guide..scrbl).'._.'.Channels.'))", "../guide/concurrency.html#%28part._.Channels%29"],
  ["(part._(.'(lib._scribblings/guide/guide..scrbl).'._.'.Class_.Contracts.'))", "../guide/classes.html#%28part._.Class_.Contracts%29"],
  ["(part._(.'(lib._scribblings/guide/guide..scrbl).'._.'.Definitions.'))", "../guide/syntax-overview.html#%28part._.Definitions%29"],
@@ -614,6 +620,7 @@ link_targets[2] = [
  ["(part._(.'(lib._scribblings/reference/reference..scrbl).'._.'print-hashtable.'))", "../reference/printing.html#%28part._print-hashtable%29"],
  ["(part._(.'(lib._scribblings/reference/reference..scrbl).'._.'print-vectors.'))", "../reference/printing.html#%28part._print-vectors%29"],
  ["(part._(.'(lib._scribblings/reference/reference..scrbl).'._.'quote.'))", "../reference/quote.html"],
+ ["(part._(.'(lib._scribblings/reference/reference..scrbl).'._.'repl-module.'))", "../reference/repl-module.html"],
  ["(part._(.'(lib._scribblings/reference/reference..scrbl).'._.'running-sa.'))", "../reference/running-sa.html"],
  ["(part._(.'(lib._scribblings/reference/reference..scrbl).'._.'special-comments.'))", "../reference/special-comments.html"],
  ["(part._(.'(lib._scribblings/reference/reference..scrbl).'._.'struct-generics.'))", "../reference/struct-generics.html"],
@@ -693,7 +700,6 @@ link_targets[2] = [
  ["(part._(.'(lib._srfi/scribblings/srfi..scrbl).'._.'srfi-27.'))", "../srfi/srfi-27.html"],
  ["(part._(.'(lib._srfi/scribblings/srfi..scrbl).'._.'srfi-34.'))", "../srfi/srfi-34.html"],
  ["(part._(.'(lib._srfi/scribblings/srfi..scrbl).'._.'srfi-45.'))", "../srfi/srfi-45.html"],
- ["(part._(.'(lib._srfi/scribblings/srfi..scrbl).'._.'srfi-5.'))", "../srfi/srfi-5.html"],
  ["(part._(.'(lib._srfi/scribblings/srfi..scrbl).'._.'srfi-63.'))", "../srfi/srfi-63.html"],
  ["(part._(.'(lib._srfi/scribblings/srfi..scrbl).'._.'srfi-74.'))", "../srfi/srfi-74.html"],
  ["(part._(.'(lib._srfi/scribblings/srfi..scrbl).'._.'srfi-9.'))", "../srfi/srfi-9.html"],
@@ -711,6 +717,7 @@ link_targets[2] = [
  ["(part._(.'(lib._syntax/scribblings/syntax..scrbl).'._.'module-reader.'))", "../syntax/reader-helpers.html#%28part._module-reader%29"],
  ["(part._(.'(lib._syntax/scribblings/syntax..scrbl).'._.'stxkeyword.'))", "../syntax/syntax-helpers.html#%28part._stxkeyword%29"],
  ["(part._(.'(lib._syntax/scribblings/syntax..scrbl).'._.'stxparse.'))", "../syntax/stxparse.html"],
+ ["(part._(.'(lib._syntax/scribblings/syntax..scrbl).'._.'syntax/apply-transformer.'))", "../syntax/transformer-helpers.html#%28part._syntax%2Fapply-transformer%29"],
  ["(part._(.'(lib._syntax/scribblings/syntax..scrbl).'._.'template.'))", "../syntax/transformer-helpers.html#%28part._template%29"],
  ["(part._(.'(lib._syntax/scribblings/syntax..scrbl).'._.'varied-meanings.'))", "../syntax/varied-meanings.html"],
  ["(part._(.'(lib._teachpack/teachpack..scrbl).'._.'.Understanding_a_.Door.'))", "../teachpack/world.html#%28part._.Understanding_a_.Door%29"],
